#!/bin/bash
# compiles all nnet libraries and test programs                             *
#                                                                           *
# Copyright 2003 by Rosario Piro (mailto: devfaust@users.sourceforge.net)   *
#                                                                           *
#   This program is free software; you can redistribute it and/or modify    *
#   it under the terms of the GNU General Public License as published by    *
#   the Free Software Foundation; either version 2 of the License, or       *
#   (at your option) any later version.                                     *
#                                                                           *
#   This program is distributed in the hope that it will be useful,         *
#   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
#   GNU General Public License for more details.                            *
#                                                                           *
#   You should have received a copy of the GNU General Public License       *
#   along with this program; if not, write to the Free Software             *
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
#   or see http://www.gnu.org/copyleft/gpl.html                             *
# ------------------------------------------------------------------------- */

VERSION="v0.1.0"

NNET_VERSION="v0.1.0"

# c++ compiler
CC="gcc"
# compiler arguments
CCARGS="-Wall -I./include -DNNET_VERSION=\"$NNET_VERSION\""
CCARGS_LIB="$CCARGS"
CCARGS_EXEC="$CCARGS -lstdc++ -L./lib"
LIBS="-lextstd -lnnet"
LIBS_FILE="$LIBS -lnnetfileio"

#if command line arguments have been specified:
if [ $# -ge 1 ]; then 

    # help screen
    if [ "$1" == "--help" ] || [ "$1" == "-h" ]; then
	echo "nnet library compilation script $VERSION"
	echo "(c) 2003 Rosario M. Piro (mailto:devfaust@users.sourceforge.net)"
	echo "published under the terms of the GNU General Public License"
	echo "(see http://www.gnu.org/copyleft/gpl.html)"
	echo ""
	echo "Usage: `basename $0` [OPTIONS]"
	echo ""
	echo "OPTIONS are:  --debug   print debugging information to sdterr"
	echo "                        while library is running, compile with"
	echo "                        debugging information (-g flag)."
	echo "                        ATTENTION: this causes a lot of output."
	echo "                        Use --debug only for small neural networks!"
	echo "              --test    compile testprogramms as well."
	echo ""
	echo "Libraries will only be compiled, not installed!"
	exit 1
    fi

    for((; 1<=$#; )); do
	if [ "$1" == "--test" ]; then
	    NNET_TEST=1
	    shift
	else
	    NNET_TEST=0
	fi
	if [ "$1" == "--debug" ]; then
	    CCARGS_LIB="$CCARGS_LIB -g -DNNET_DEBUG"
	    CCARGS_EXEC="$CCARGS_EXEC -g -DNNET_DEBUG"
	    shift
	fi
    done
fi

echo "nnet library compilation script $VERSION"
echo "(c) 2003 Rosario M. Piro (mailto:devfaust@users.sourceforge.net)"
echo "published under the terms of the GNU General Public License"
echo "(see http://www.gnu.org/copyleft/gpl.html)"
echo ""

echo "compiling libraries:"

# libextstd.a
echo "$CC $CCARGS_LIB -c -o lib/src/extstdlib.o lib/src/extstdlib.c"
$CC $CCARGS_LIB -c -o lib/src/extstdlib.o lib/src/extstdlib.c
echo "ar rcs lib/libextstd.a lib/src/extstdlib.o"
ar rcs lib/libextstd.a lib/src/extstdlib.o
echo "rm -vf lib/src/extstdlib.o"
rm -vf lib/src/extstdlib.o
echo ""

# libnnet.a
echo "$CC $CCARGS_LIB -c -o lib/src/nnet.o lib/src/nnet.cpp"
$CC $CCARGS_LIB -c -o lib/src/nnet.o lib/src/nnet.cpp
echo "ar rcs lib/libnnet.a lib/src/nnet.o"
ar rcs lib/libnnet.a lib/src/nnet.o
echo "rm -vf lib/src/nnet.o"
rm -vf lib/src/nnet.o
echo ""

# libnnetfileio.a
echo "$CC $CCARGS_LIB -c -o lib/src/nnetfileio.o lib/src/nnetfileio.cpp"
$CC $CCARGS_LIB -c -o lib/src/nnetfileio.o lib/src/nnetfileio.cpp
echo "ar rcs lib/libnnetfileio.a lib/src/nnetfileio.o"
ar rcs lib/libnnetfileio.a lib/src/nnetfileio.o
echo "rm -vf lib/src/nnetfileio.o"
rm -vf lib/src/nnetfileio.o
echo ""

if [ $NNET_TEST == 1 ]; then

    echo "compiling test programs:"

    # test_nodes
    echo "compiling test program for basic nodes Neuron and Synpase:"
    echo "$CC $CCARGS_EXEC -o bin/test_nodes src/test_nodes.cpp $LIBS"
    $CC $CCARGS_EXEC -o bin/test_nodes src/test_nodes.cpp $LIBS
    echo ""

    # test_nnet
    echo "compiling test program for network structures NNet, FFNNet, FFNNetBP:"
    echo "$CC $CCARGS_EXEC -o bin/test_nnet src/test_nnet.cpp $LIBS"
    $CC $CCARGS_EXEC -o bin/test_nnet src/test_nnet.cpp $LIBS
    echo ""

    # test_file
    echo "compiling test program for network file I/O:"
    echo "$CC $CCARGS_EXEC -o bin/test_file src/test_file.cpp $LIBS -include lib/src/nnetfileio.cpp"
    $CC $CCARGS_EXEC -o bin/test_file src/test_file.cpp $LIBS -include lib/src/nnetfileio.cpp
    echo ""
fi

echo "done."
