/*****************************************************************************
 * nnetfileio.hpp - Header file for loading/saving neural networks (C++)     *
 * (libnnetfileio.so) or (libnnetfileio.a) - Version 0.1                     *
 * ------------------------------------------------------------------------- *
 * This library contains functions that load or save neural networks         *
 * based on the libnnet library.                                             *
 * ------------------------------------------------------------------------- *
 * Copyright 2003 by Rosario Piro (mailto: devfaust@users.sourceforge.net)   *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the Free Software             *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 *   or see http://www.gnu.org/copyleft/gpl.html                             *
 * ------------------------------------------------------------------------- */

#ifndef _NNETFILEIO_H_
#define _NNETFILEIO_H_

//*********************
//*** include files ***
//*********************
#include <nnet.hpp>
#include <string>

namespace nnet {

  // the following functions currently support objects of types
  // NNet, FFNNet and FFNNetBP.

  // SaveNNet() saves the current state of the nnet to a file
  // returns true if successfully saved.
  inline bool SaveNNet(const NNet &net, const string filename);
  inline bool SaveNNet(const FFNNet &net, const string filename);
  inline bool SaveNNet(const FFNNetBP &net, const string filename);

  // LoadNNet() loads a nnet from a file, and returns a pointer to it (NULL
  // if an error occured). The user eventually has to typecast the pointer
  // in order to access members of classes that are derived from NNet.
  NNet *LoadNNet(const string filename);

} // namespace nnet

#endif // #define _NNETFILEIO_H_
