/*****************************************************************************
 * stdclassdef.hpp - Header file for standard class definitions (C++)        *
 * Version 0.0.1                                                             *
 * ------------------------------------------------------------------------- *
 * This header file contains preprocessor macros to simplify standard class  *
 * code ...                                                                  *
 * ------------------------------------------------------------------------- *
 * Copyright 2003 by Rosario Piro (mailto: devfaust@users.sourceforge.net)   *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the Free Software             *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 *   or see http://www.gnu.org/copyleft/gpl.html                             *
 * ------------------------------------------------------------------------- */

#ifndef _STDCLASSDEF_LIB_
#define _STDCLASSDEF_LIB_

//***************************
//*** preprocessor macros ***
//***************************

// *** The following macros have to inculded into the class definition! ***
//
// for private data members:
// PRIV_ATTRIBUTE_GET_SET(memberType, memberName, publicName)
// PRIV_ATTRIBUTE_GET(memberType, memberName, publicName)
// PRIV_ATTRIBUTE_SET(memberType, memberName, publicName)
// for private data members, call by reference:
// PRIV_ATTRIBUTE_GET_SET_REF(memberType, memberName, publicName)
// PRIV_ATTRIBUTE_GET_REF(memberType, memberName, publicName)
// PRIV_ATTRIBUTE_SET_REF(memberType, memberName, publicName)
// 
// for protected data members:
// PROT_ATTRIBUTE_GET_SET(memberType, memberName, publicName)
// PROT_ATTRIBUTE_GET(memberType, memberName, publicName)
// PROT_ATTRIBUTE_SET(memberType, memberName, publicName)
// for protected  data members, call by reference:
// PROT_ATTRIBUTE_GET_SET_REF(memberType, memberName, publicName)
// PROT_ATTRIBUTE_GET_REF(memberType, memberName, publicName)
// PROT_ATTRIBUTE_SET_REF(memberType, memberName, publicName)


// these macros add a PRIVATE data member including it's standard accessor
// methods. example: PRIV_ATTRIBUTE_GET_SET(int, _value, Value) within the
// class body will add the private member "int _value" and the public methods
// "int getValue()" and "void setValue(int)" ... 
#define PRIV_ATTRIBUTE_GET_SET(memberType, memberName, publicName)     \
  private:                                                             \
     memberType memberName;                                            \
  public:                                                              \
     memberType Get##publicName() const {                              \
        return memberName;                                             \
     }                                                                 \
     void Set##publicName(memberType new_##memberName) {               \
        memberName = new_##memberName;                                 \
     }
#define PRIV_ATTRIBUTE_GET(memberType, memberName, publicName)         \
  private:                                                             \
     memberType memberName;                                            \
  public:                                                              \
     memberType Get##publicName() const {                              \
        return memberName;                                             \
     }
#define PRIV_ATTRIBUTE_SET(memberType, memberName, publicName)         \
  private:                                                             \
     memberType memberName;                                            \
  public:                                                              \
     void Set##publicName(memberType new_##memberName) {               \
        memberName = new_##memberName;                                 \
     }

// the following macros add a PRIVATE data member (as above) which's accessor
// methods return/accept references instead of objects ...
#define PRIV_ATTRIBUTE_GET_SET_REF(memberType, memberName, publicName) \
  private:                                                             \
     memberType memberName;                                            \
  public:                                                              \
     memberType &Get##publicName() {                                   \
        return memberName;                                             \
     }                                                                 \
     void Set##publicName(const memberType &new_##memberName) {        \
        memberName = new_##memberName;                                 \
     }
#define PRIV_ATTRIBUTE_GET_REF(memberType, memberName, publicName)     \
  private:                                                             \
     memberType memberName;                                            \
  public:                                                              \
     memberType &Get##publicName() {                                   \
        return memberName;                                             \
     }
#define PRIV_ATTRIBUTE_SET_REF(memberType, memberName, publicName)     \
  private:                                                             \
     memberType memberName;                                            \
  public:                                                              \
     void Set##publicName(const memberType &new_##memberName) {        \
        memberName = new_##memberName;                                 \
     }


// these macros add a PROTECTED data member including it's standard accessor
// methods. example: PROT_ATTRIBUTE_GET_SET(int, _value, Value) within the
// class body will add the private member "int _value" and the public methods
// "int getValue()" and "void setValue(int )" ... 
#define PROT_ATTRIBUTE_GET_SET(memberType, memberName, publicName)     \
  protected:                                                           \
     memberType memberName;                                            \
  public:                                                              \
     memberType Get##publicName() const {                              \
        return memberName;                                             \
     }                                                                 \
     void Set##publicName(memberType new_##memberName) {               \
        memberName = new_##memberName;                                 \
     }
#define PROT_ATTRIBUTE_GET(memberType, memberName, publicName)         \
  protected:                                                           \
     memberType memberName;                                            \
  public:                                                              \
     memberType Get##publicName() const {                              \
        return memberName;                                             \
     }
#define PROT_ATTRIBUTE_SET(memberType, memberName, publicName)         \
  protected:                                                           \
     memberType memberName;                                            \
  public:                                                              \
     void Set##publicName(memberType new_##memberName) {               \
        memberName = new_##memberName;                                 \
     }


// the following macros add a PROTECTED data member (as above) which's
// accessor methods return/accept references instead of objects ...
#define PROT_ATTRIBUTE_GET_SET_REF(memberType, memberName, publicName) \
  protected:                                                           \
     memberType memberName;                                            \
  public:                                                              \
     memberType &Get##publicName() {                                   \
        return memberName;                                             \
     }                                                                 \
     void Set##publicName(const memberType &new_##memberName) {        \
        memberName = new_##memberName;                                 \
     }
#define PROT_ATTRIBUTE_GET_REF(memberType, memberName, publicName)     \
  protected:                                                           \
     memberType memberName;                                            \
  public:                                                              \
     memberType &Get##publicName() {                                   \
        return memberName;                                             \
     }
#define PROT_ATTRIBUTE_SET_REF(memberType, memberName, publicName)     \
  protected:                                                           \
     memberType memberName;                                            \
  public:                                                              \
     void Set##publicName(const memberType &new_##memberName) {        \
        memberName = new_##memberName;                                 \
     }


#endif // #ifndef _STDCLASSDEF_LIB_
