/*****************************************************************************
 * test_nodes.cpp - test program for basic neural network library (C++)      *
 * (libnnet.so) or (libnnet.a) - Version 0.1                                 *
 * ------------------------------------------------------------------------- *
 * This program tests the library containing the basic classes and methods   *
 * for the implementation of simple neural networks. It tests the            *
 * the functionality of the node classes Neuron and Synapse.                 *
 * ------------------------------------------------------------------------- *
 * Copyright 2003 by Rosario Piro (mailto: devfaust@users.sourceforge.net)   *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the Free Software             *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 *   or see http://www.gnu.org/copyleft/gpl.html                             *
 * ------------------------------------------------------------------------- */

#include <nnet.hpp>
#include <iostream>

using namespace nnet;

int main() {

  vector<InputTerminal *> IT; // still empty.
  vector<Neuron *> Layer1, Layer2;

  InputTerminal *newIT;
  Neuron *newLay1, *newLay2;

  for(int id = 1; id <= 10; id++) {
    // create a new InputTerminal with NeuronID=id
    newIT = new InputTerminal(id);
    IT.push_back(newIT);
    cout << "IT[" << id-1 <<"], NeuronID=" << IT[id-1]->GetNeuronID() 
	 << " on memory address " << IT[id-1] << endl;
    // create neuron for first layer.
    newLay1 = new Neuron(id+100);
    Layer1.push_back(newLay1);
    cout << "Layer1[" << id-1 <<"], NeuronID=" << Layer1[id-1]->GetNeuronID()
	 << " on memory address " << Layer1[id-1] << endl;
    // create neuron for second layer.
    newLay2 = new Neuron(id+200);
    Layer2.push_back(newLay2);
    cout << "Layer2[" << id-1 <<"], NeuronID=" << Layer2[id-1]->GetNeuronID()
	 << " on memory address " << Layer2[id-1] << endl;
  }


  int synID = 1000;
  for(int it = 0; it < 10; it++) {
    // connecting input terminals to layer1
    for(int l1n = 0; l1n < 10; l1n++) {
      synID++;
      IT[it]->AddOutNeuron(*Layer1[l1n], synID);
    }

    // the following part is more than rubbish but it did help me to fix a bug
    // due to insufficient memory allocation of an implicit creation of an
    // vector<Synapsis *> object ... I still don't know why it happened, but
    // I assume it was my own stupidity instead of a STL bug ;o)
    // The bug however did cause trouble with an iterator for the obect.
    // Now I declared a pointer to a vector<Synapsis *> object as a data member
    // of neutron and allocate the necessary memory explicitly in the class
    // constructor ...
    Neuron *nptr = IT[it]->GetOutSynVect().front()->GetFrom();
    cout << "from ptr: " << nptr << endl;
    vector<Synapse *>::const_iterator iter
      = nptr->GetOutSynVect().begin();
    cout << "should be: " << *(nptr->GetOutSynVect().begin()) << endl;
    cout << "but is: " << *(IT[it]->GetOutSynVect().begin()) << endl;
    cout << "but is: " << *iter << endl;
    for(; iter!=nptr->GetOutSynVect().end(); iter++)
      cout << (*iter) << "; " << endl;
    cout << endl;
  }


  synID = 2000;
  for(int l1n = 0; l1n < 10; l1n++) {
    // connecting layer1 neurons to layer2
    for(int l2n = 0; l2n < 10; l2n++) {
      synID++;
      Layer2[l2n]->AddInNeuron(*Layer1[l1n], synID);
    }
  }


  cout << "SynID of IT[3] to Layer1[6] = "
       << (IT[3]->GetOutSynVect())[6]->GetSynID() << endl;
  cout << "NeuronID of Layer2[2] = "
       << ((IT[3]->GetOutSynVect())[6]->GetTo()->GetOutSynVect())[2]->GetTo()->GetNeuronID() << flush << endl;

  cout << "synapse IT[0], NeuronID=" << IT[0]->GetNeuronID() << " -> Layer1[0], NeuronID=" << Layer1[0]->GetNeuronID() << " has synID=" << IT[0]->GetOutSynVect().front()->GetSynID() << endl;
  cout << "synapse IT[0], NeuronID=" << (*IT.begin())->GetNeuronID() << " -> Layer1[0], NeuronID=" << (*Layer1.begin())->GetNeuronID() << " has synID=" << ((*(*IT.begin())->GetOutSynVect().begin()))->GetSynID() << endl;

  cout << "Set all inputs to 1.0 ... ";
  for (vector<InputTerminal *>::iterator iter=IT.begin(); iter!=IT.end(); iter++)
    (*iter)->SetInput(1.0);
  cout << " done!" << flush << endl;
  cout << "WeightedVal of IT[3]._outSynVect[6] = " << (IT[3]->GetOutSynVect())[6]->WeightedVal() << flush << endl;

  cout << "Now get AxonOutput of Layer1: " << endl;
  for (vector<Neuron *>::iterator iter=Layer1.begin(); iter!=Layer1.end(); iter++)
    cout << "NeuronID=" << (*iter)->GetNeuronID() << ", output is "
	 << (*iter)->AxonOutput() << endl;

  cout << "Now get AxonOutput of Layer2: " << endl;
  for (vector<Neuron *>::iterator iter=Layer2.begin(); iter!=Layer2.end(); iter++)
    cout << "NeuronID=" << (*iter)->GetNeuronID() << ", output is "
	 << (*iter)->AxonOutput() << endl;

  cout << "_inChange of Layer1[3] is " << Layer1[3]->GetInputChange() << " and Layer2[8] is " << Layer2[8]->GetInputChange() << endl;
  cout << "Weight of syn Layer1[3]->Layer2[8] is " << (Layer1[3]->GetOutSynVect())[8]->GetWeight() << endl;
  cout << "now set new weight of syn Layer1[3]->Layer2[8]: new value is" << endl;
  (Layer1[3]->GetOutSynVect())[8]->SetWeight(0.234);
  cout << (Layer1[3]->GetOutSynVect())[8]->GetWeight() << endl;
  cout << "_isChange of Layer2[8] is now " << Layer2[8]->GetInputChange() << endl;

  int lastSynID;
  Synapse *synAddr;
  cout << "The following synapses are input synapses of Layer1[2]: " << endl;
  for (synID=0; synID<=3000; synID++) 
    if( (synAddr = Layer1[2]->GetInSyn(synID)) ) {
      cout << "SynID=" << synID << ", address " << synAddr << ", is ";
      if(!Layer1[2]->IsInSyn(synAddr)) cout << "NOT ";
      cout << "input." << endl;
      lastSynID = synID;
    }
  cout << "Deleting last synapse with synID=" << lastSynID << endl;
  Layer1[2]->DelInSyn(lastSynID);
  cout << "NeuronID=10 has now " << IT[9]->GetOutSynVect().size()
       << " output synapses (should be 9)" << endl;
  cout << "Now input synapses of Layer1[2]: " << endl;
  for (synID=0; synID<=3000; synID++) 
    if( (synAddr = Layer1[2]->GetInSyn(synID)) ) {
      cout << "SynID=" << synID << ", address " << synAddr << ", is ";
      if(!Layer1[2]->IsInSyn(synAddr)) cout << "NOT ";
      cout << "input." << endl;
    }

  cout << "The following synapses are output synapses of Layer1[3]: " << endl;
  for (synID=0; synID<=3000; synID++) 
    if( (synAddr = Layer1[3]->GetOutSyn(synID)) ) {
      cout << "SynID=" << synID << ", address " << synAddr << ", is ";
      if(!Layer1[3]->IsOutSyn(synAddr)) cout << "NOT ";
      cout << "output." << endl;
      lastSynID = synID;
    }
  cout << "Deleting last synapse with synID=" << lastSynID << endl;
  Layer1[3]->DelOutSyn(lastSynID);
  cout << "NeuronID=210 has now " << Layer2[9]->GetInSynVect().size()
       << " input synapses (should be 9)" << endl;
  cout << "Now output synapses of Layer1[3]: " << endl;
  for (synID=0; synID<=3000; synID++) 
    if( (synAddr = Layer1[3]->GetOutSyn(synID)) ) {
      cout << "SynID=" << synID << ", address " << synAddr  << ", is ";
      if(!Layer1[3]->IsOutSyn(synAddr)) cout << "NOT ";
      cout << "output." << endl;
    }

  cout << "The following synapses are output synapses of Layer2[3]: " << endl
       << "(it's an output terminal there should be no outgoing synapses!)"
       << endl;
  for (synID=0; synID<=3000; synID++) 
    if( (synAddr = Layer2[3]->GetOutSyn(synID)) ) {
      cout << "SynID=" << synID << ", address " << synAddr  << ", is ";
      if(!Layer1[3]->IsOutSyn(synAddr)) cout << "NOT ";
      cout << "output." << endl;
      lastSynID = synID;
    }

  Synapse *newIOSyn = new Synapse(9999);
  // test AddInSyn()
  cout <<endl<< "Adding an additional input synapse with synID=9999, address=" 
       << newIOSyn << endl << "(from=" << newIOSyn->GetFrom() << ", to="
       << newIOSyn->GetTo() << ")" << endl 
       << "to NeuronID=" << Layer1[4]->GetNeuronID()
       << ", address=" << Layer1[4]->GetInSynVect().front()->GetTo() << endl;
  Layer1[4]->AddInSyn(newIOSyn);
  // check
  cout << "New size of _inSynVect should be 11, is "
       <<  Layer1[4]->GetInSynVect().size() << endl;
  if(Layer1[4]->IsInSyn(newIOSyn))
    cout << "SynID=9999 is input! check ok!" << endl;
  else
    cout << "SynID=9999 is NOT input! error!" << endl;
  cout << "The synapse's connections are now: from="
       << newIOSyn->GetFrom() << ", to=" << newIOSyn->GetTo() << endl;
  // test AddOutSyn()
  cout <<endl<<"Adding an additional output synapse with synID=9999, address=" 
       << newIOSyn << endl << "(from=" << newIOSyn->GetFrom() << ", to="
       << newIOSyn->GetTo() << ")" << endl 
       << "to NeuronID=" << Layer2[7]->GetNeuronID()
       << ", address=" << Layer2[7]->GetInSynVect().front()->GetTo() << endl;
  Layer2[7]->AddOutSyn(newIOSyn);
  // check
  cout << "New size of _outSynVect should be 1, is "
       <<  Layer2[7]->GetOutSynVect().size() << endl;
  if(Layer2[7]->IsOutSyn(newIOSyn))
    cout << "SynID=9999 is input! check ok!" << endl;
  else
    cout << "SynID=9999 is NOT input! error!" << endl;
  cout << "The synapse's connections are now: from="
       << newIOSyn->GetFrom() << ", to=" << newIOSyn->GetTo() << endl;

  cout << endl << "Now check the activation function and it's derivate:"
       << endl;

  InputTerminal test1(1);
  Neuron test2(2);
  test1.AddOutNeuron(test2, 12);
  test1.GetOutSynVect().front()->SetWeight(1);
  test1.SetInput(.5);
  cout << endl << "ActFunction(0.5) should return 0.2449186624, returns: "
       << test2.AxonOutput() << endl;

  cout << "DerivActFunction(0.5) should return 0.4700074244, returns: "
       << test2.DerivActFunction() << endl << endl;

  cout << "Finally we have to check both functionallities of the Delta()"
       << " function:" << endl << "If error is 9.14879, then " << endl;

  double error = 9.14879;
  cout << "error * DerivActFunction(0.5) = 4.29999922428" << endl
       << "Delta(error) = error * DerivActFunction(0.5) = "
       << test2.Delta(error) << endl;

  Neuron test3(3), test4(4);
  test2.AddOutNeuron(test3, 23);
  test2.GetOutSynVect().front()->SetWeight(0.5);
  test2.AddOutNeuron(test4, 24);
  test2.GetOutSynVect().back()->SetWeight(1.5);
  test2.AxonOutput();
  cout << "test3.AxonOutput() should be 0.061154, is " << test3.AxonOutput()
       << endl;
  cout << "test4.AxonOutput() should be 0.181652, is " << test4.AxonOutput()
       << endl;
  test3.Delta(6.4);
  test4.Delta(0.3);
  cout << "test2.Delta() should be 0.851460 ..." << endl;
  test2.Delta();

  cout << "Ok, Chummer! NNet node test completed." << endl;

  return 0;
}
